@@ -6,6 +6,7 @@
  */
 
 #include "iterator.h"
+
 #include "tree.h"
 #include "index.h"
 
@@ -1296,11 +1297,6 @@ static filesystem_iterator_entry *filesystem_iterator_entry_init(
 	return entry;
 }
 
-#ifdef _WIN32
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat"
-#endif
-
 static int filesystem_iterator_frame_push(
 	filesystem_iterator *iter,
 	filesystem_iterator_entry *frame_entry)
@@ -1443,10 +1439,6 @@ done:
 	return error;
 }
 
-#ifdef _WIN32
-#pragma GCC diagnostic pop
-#endif
-
 GIT_INLINE(void) filesystem_iterator_frame_pop(filesystem_iterator *iter)
 {
 	filesystem_iterator_frame *frame;
